@@ -1,23 +1,17 @@
 import os
+from distutils.util import strtobool
 
 __all__ = [
     'keras', 'utils', 'activations', 'applications', 'backend', 'datasets', 'engine',
     'layers', 'preprocessing', 'wrappers', 'callbacks', 'constraints', 'initializers',
-    'metrics', 'models', 'losses', 'optimizers', 'regularizers', 'TF_KERAS', 'EAGER_MODE'
+    'metrics', 'models', 'losses', 'optimizers', 'regularizers', 'TF_KERAS',
 ]
 
-TF_KERAS = False
-EAGER_MODE = False
+TF_KERAS = strtobool(os.environ.get('TF_KERAS', '0'))
 
-if 'TF_KERAS' in os.environ and os.environ['TF_KERAS'] != '0':
+if TF_KERAS:
     import tensorflow as tf
-    from tensorflow.python import keras
-    TF_KERAS = True
-    if 'TF_EAGER' in os.environ and os.environ['TF_EAGER'] != '0':
-        if int(tf.version.VERSION.split('.')[0]) < 2:
-            import tensorflow as tf
-            tf.enable_eager_execution()
-        EAGER_MODE = True
+    keras = tf.keras
 else:
     import keras
 
@@ -26,7 +20,6 @@ activations = keras.activations
 applications = keras.applications
 backend = keras.backend
 datasets = keras.datasets
-engine = keras.engine
 layers = keras.layers
 preprocessing = keras.preprocessing
 wrappers = keras.wrappers
