@@ -24,7 +24,14 @@ def get_custom_objects():
     }
 
 
-def _wrap_layer(name, input_layer, build_func, dropout_rate=0.0, trainable=True):
+def _wrap_layer(name,
+                input_layer,
+                build_func,
+                dropout_rate=0.0,
+                trainable=True,
+                use_adapter=False,
+                adapter_units=None,
+                adapter_activation='relu'):
     """Wrap layers with residual, normalization and dropout.
 
     :param name: Prefix of names for internal layers.
@@ -32,6 +39,9 @@ def _wrap_layer(name, input_layer, build_func, dropout_rate=0.0, trainable=True)
     :param build_func: A callable that takes the input tensor and generates the output tensor.
     :param dropout_rate: Dropout rate.
     :param trainable: Whether the layers are trainable.
+    :param use_adapter: Whether to use feed-forward adapters before each residual connections.
+    :param adapter_units: The dimension of the first transformation in feed-forward adapter.
+    :param adapter_activation: The activation after the first transformation in feed-forward adapter.
     :return: Output layer.
     """
     build_output = build_func(input_layer)
@@ -44,6 +54,14 @@ def _wrap_layer(name, input_layer, build_func, dropout_rate=0.0, trainable=True)
         dropout_layer = build_output
     if isinstance(input_layer, list):
         input_layer = input_layer[0]
+    if use_adapter:
+        adapter = FeedForward(
+            units=adapter_units,
+            activation=adapter_activation,
+            kernel_initializer=keras.initializers.TruncatedNormal(mean=0.0, stddev=0.001),
+            name='%s-Adapter' % name,
+        )(dropout_layer)
+        dropout_layer = keras.layers.Add(name='%s-Adapter-Add' % name)([dropout_layer, adapter])
     add_layer = keras.layers.Add(name='%s-Add' % name)([input_layer, dropout_layer])
     normal_layer = LayerNormalization(
         trainable=trainable,
@@ -52,7 +70,11 @@ def _wrap_layer(name, input_layer, build_func, dropout_rate=0.0, trainable=True)
     return normal_layer
 
 
-def attention_builder(name, head_num, activation, history_only, trainable=True):
+def attention_builder(name,
+                      head_num,
+                      activation,
+                      history_only,
+                      trainable=True):
     """Get multi-head self-attention builder.
 
     :param name: Prefix of names for internal layers.
@@ -73,7 +95,10 @@ def attention_builder(name, head_num, activation, history_only, trainable=True):
     return _attention_builder
 
 
-def feed_forward_builder(name, hidden_dim, activation, trainable=True):
+def feed_forward_builder(name,
+                         hidden_dim,
+                         activation,
+                         trainable=True):
     """Get position-wise feed-forward layer builder.
 
     :param name: Prefix of names for internal layers.
@@ -99,7 +124,10 @@ def get_encoder_component(name,
                           attention_activation=None,
                           feed_forward_activation='relu',
                           dropout_rate=0.0,
-                          trainable=True):
+                          trainable=True,
+                          use_adapter=False,
+                          adapter_units=None,
+                          adapter_activation='relu'):
     """Multi-head self-attention and feed-forward layer.
 
     :param name: Prefix of names for internal layers.
@@ -110,6 +138,9 @@ def get_encoder_component(name,
     :param feed_forward_activation: Activation for feed-forward layer.
     :param dropout_rate: Dropout rate.
     :param trainable: Whether the layers are trainable.
+    :param use_adapter: Whether to use feed-forward adapters before each residual connections.
+    :param adapter_units: The dimension of the first transformation in feed-forward adapter.
+    :param adapter_activation: The activation after the first transformation in feed-forward adapter.
     :return: Output layer.
     """
     attention_name = '%s-MultiHeadSelfAttention' % name
@@ -126,6 +157,9 @@ def get_encoder_component(name,
         ),
         dropout_rate=dropout_rate,
         trainable=trainable,
+        use_adapter=use_adapter,
+        adapter_units=adapter_units,
+        adapter_activation=adapter_activation,
     )
     feed_forward_layer = _wrap_layer(
         name=feed_forward_name,
@@ -138,6 +172,9 @@ def get_encoder_component(name,
         ),
         dropout_rate=dropout_rate,
         trainable=trainable,
+        use_adapter=use_adapter,
+        adapter_units=adapter_units,
+        adapter_activation=adapter_activation,
     )
     return feed_forward_layer
 
@@ -150,7 +187,10 @@ def get_decoder_component(name,
                           attention_activation=None,
                           feed_forward_activation='relu',
                           dropout_rate=0.0,
-                          trainable=True):
+                          trainable=True,
+                          use_adapter=False,
+                          adapter_units=None,
+                          adapter_activation='relu'):
     """Multi-head self-attention, multi-head query attention and feed-forward layer.
 
     :param name: Prefix of names for internal layers.
@@ -162,6 +202,9 @@ def get_decoder_component(name,
     :param feed_forward_activation: Activation for feed-forward layer.
     :param dropout_rate: Dropout rate.
     :param trainable: Whether the layers are trainable.
+    :param use_adapter: Whether to use feed-forward adapters before each residual connections.
+    :param adapter_units: The dimension of the first transformation in feed-forward adapter.
+    :param adapter_activation: The activation after the first transformation in feed-forward adapter.
     :return: Output layer.
     """
     self_attention_name = '%s-MultiHeadSelfAttention' % name
@@ -179,6 +222,9 @@ def get_decoder_component(name,
         ),
         dropout_rate=dropout_rate,
         trainable=trainable,
+        use_adapter=use_adapter,
+        adapter_units=adapter_units,
+        adapter_activation=adapter_activation,
     )
     query_attention_layer = _wrap_layer(
         name=query_attention_name,
@@ -192,6 +238,9 @@ def get_decoder_component(name,
         ),
         dropout_rate=dropout_rate,
         trainable=trainable,
+        use_adapter=use_adapter,
+        adapter_units=adapter_units,
+        adapter_activation=adapter_activation,
     )
     feed_forward_layer = _wrap_layer(
         name=feed_forward_name,
@@ -204,6 +253,9 @@ def get_decoder_component(name,
         ),
         dropout_rate=dropout_rate,
         trainable=trainable,
+        use_adapter=use_adapter,
+        adapter_units=adapter_units,
+        adapter_activation=adapter_activation,
     )
     return feed_forward_layer
 
@@ -215,7 +267,10 @@ def get_encoders(encoder_num,
                  attention_activation=None,
                  feed_forward_activation='relu',
                  dropout_rate=0.0,
-                 trainable=True):
+                 trainable=True,
+                 use_adapter=False,
+                 adapter_units=None,
+                 adapter_activation='relu'):
     """Get encoders.
 
     :param encoder_num: Number of encoder components.
@@ -226,6 +281,9 @@ def get_encoders(encoder_num,
     :param feed_forward_activation: Activation for feed-forward layer.
     :param dropout_rate: Dropout rate.
     :param trainable: Whether the layers are trainable.
+    :param use_adapter: Whether to use feed-forward adapters before each residual connections.
+    :param adapter_units: The dimension of the first transformation in feed-forward adapter.
+    :param adapter_activation: The activation after the first transformation in feed-forward adapter.
     :return: Output layer.
     """
     last_layer = input_layer
@@ -239,6 +297,9 @@ def get_encoders(encoder_num,
             feed_forward_activation=feed_forward_activation,
             dropout_rate=dropout_rate,
             trainable=trainable,
+            use_adapter=use_adapter,
+            adapter_units=adapter_units,
+            adapter_activation=adapter_activation,
         )
     return last_layer
 
@@ -251,7 +312,10 @@ def get_decoders(decoder_num,
                  attention_activation=None,
                  feed_forward_activation='relu',
                  dropout_rate=0.0,
-                 trainable=True):
+                 trainable=True,
+                 use_adapter=False,
+                 adapter_units=None,
+                 adapter_activation='relu'):
     """Get decoders.
 
     :param decoder_num: Number of decoder components.
@@ -263,6 +327,9 @@ def get_decoders(decoder_num,
     :param feed_forward_activation: Activation for feed-forward layer.
     :param dropout_rate: Dropout rate.
     :param trainable: Whether the layers are trainable.
+    :param use_adapter: Whether to use feed-forward adapters before each residual connections.
+    :param adapter_units: The dimension of the first transformation in feed-forward adapter.
+    :param adapter_activation: The activation after the first transformation in feed-forward adapter.
     :return: Output layer.
     """
     last_layer = input_layer
@@ -277,6 +344,9 @@ def get_decoders(decoder_num,
             feed_forward_activation=feed_forward_activation,
             dropout_rate=dropout_rate,
             trainable=trainable,
+            use_adapter=use_adapter,
+            adapter_units=adapter_units,
+            adapter_activation=adapter_activation,
         )
     return last_layer
 
@@ -293,7 +363,10 @@ def get_model(token_num,
               use_same_embed=True,
               embed_weights=None,
               embed_trainable=None,
-              trainable=True):
+              trainable=True,
+              use_adapter=False,
+              adapter_units=None,
+              adapter_activation='relu'):
     """Get full model without compilation.
 
     :param token_num: Number of distinct tokens.
@@ -311,6 +384,9 @@ def get_model(token_num,
     :param embed_trainable: Whether the token embedding is trainable. It will automatically set to False if the given
                             value is None when embedding weights has been provided.
     :param trainable: Whether the layers are trainable.
+    :param use_adapter: Whether to use feed-forward adapters before each residual connections.
+    :param adapter_units: The dimension of the first transformation in feed-forward adapter.
+    :param adapter_activation: The activation after the first transformation in feed-forward adapter.
     :return: Keras model.
     """
     if not isinstance(token_num, list):
@@ -373,6 +449,9 @@ def get_model(token_num,
         feed_forward_activation=feed_forward_activation,
         dropout_rate=dropout_rate,
         trainable=trainable,
+        use_adapter=use_adapter,
+        adapter_units=adapter_units,
+        adapter_activation=adapter_activation,
     )
     decoder_input = keras.layers.Input(shape=(None,), name='Decoder-Input')
     decoder_embed, decoder_embed_weights = decoder_embed_layer(decoder_input)
@@ -390,6 +469,9 @@ def get_model(token_num,
         feed_forward_activation=feed_forward_activation,
         dropout_rate=dropout_rate,
         trainable=trainable,
+        use_adapter=use_adapter,
+        adapter_units=adapter_units,
+        adapter_activation=adapter_activation,
     )
     dense_layer = EmbeddingSim(
         trainable=trainable,
@@ -476,7 +558,8 @@ def decode(model,
             decoder_inputs[index_map[i]].append(last_token)
             if last_token == end_token or\
                     (max_len is not None and output_len >= max_len) or\
-                    _get_max_suffix_repeat_times(decoder_inputs, max_repeat * max_repeat_block) >= max_repeat:
+                    _get_max_suffix_repeat_times(decoder_inputs[index_map[i]],
+                                                 max_repeat * max_repeat_block) >= max_repeat:
                 outputs[index_map[i]] = decoder_inputs[index_map[i]]
     if is_single:
         outputs = outputs[0]
